//****************************************************************************
//
// LIB7312.H - Function prototype defintions for the library routines
//
// Copyright (c) 2001 Cirrus Logic, Inc.
//
//****************************************************************************

typedef void (*PFNISR)(void);

//
// Device Definitions for ATA devices.  Two devices are allowed.
//
#define DEVICE0 0x00
#define DEVICE1 0x10

//
// For Color LCD, width is 240 pixels, 
// and height 320 x 3 (red, green, blue).  
// Pixel depth is 4 bpp
//
#define LCD_X_SIZE		320  
#define LCD_Y_SIZE		240

//
// Definition of ColorPixel structure
//
typedef struct CLRPixel{
  char r;
  char g;
  char b;
} CPixel;

//
// From adc.c:
//
extern void ADCEnable(void);
extern void ADCDisable(void);
extern long ADCGetData(unsigned long ulADCctl);

//
// From ata.c
//
extern void ATAInitializeInterface(void);
extern unsigned char ATAReadCommandRegister(unsigned char ucRegister);
extern void ATAWriteCommandRegister(unsigned char ucRegister, unsigned char ucData);
extern unsigned char ATAReadControlRegister(unsigned char ucRegister);
extern void ATAWriteControlRegister(unsigned char ucRegister, unsigned char ucData);
extern unsigned long ATAReadDataRegister(unsigned short *pusBuffer, unsigned long ulCount);
extern unsigned long ATAWriteDataRegister(unsigned short *pusBuffer, unsigned long ulCount);
extern unsigned int ATASimpleRegPacket(int dev, unsigned int uiCommandPacketByteCount,		
 		           unsigned short cfp[6], unsigned int uiTransferDir,				
                       unsigned int uiDataPacketByteCount, 
                       unsigned short * data_pointer );
extern void ATASelectDevice (int DeviceNumber);  
extern void ATADeviceReset(unsigned char ucDevice);
extern unsigned char ATAExecuteDeviceDiagnostic(void);
extern int ATAIdentifyPacketDevice(unsigned short *);

//
// From codec.c:
//
extern void CodecEnable(void);
extern void CodecDisable(void);
extern void CodecPlay(char *pcBuffer, long lLength);
extern void CodecPlayBg(char *pcBuffer, long lLength);
extern int CodecPlaySpaceAvailable(void);
extern int CodecPlayIsDone(void);
extern void CodecPlayWaitTilDone(void);
extern void CodecRecord(char *pcBuffer, long lLength);
extern void CodecRecordBg(char *pcBuffer, long lLength);
extern int CodecRecordSpaceAvailable(void);
extern int CodecRecordIsDone(void);
extern void CodecRecordWaitTilDone(void);

//
// From ir.c:
//
extern long IREnable(long lDataRate);
extern void IRDisable(void);
extern void IRSendChar(char cChar);
extern char IRReceiveChar(void);
extern long IRCharReady(void);

//
// From isr.c:
//
extern void InterruptInstallIRQ(void);
extern void InterruptRemoveIRQ(void);
extern void InterruptInstallFIQ(void);
extern void InterruptRemoveFIQ(void);
extern void InterruptSetCodecHandler(PFNISR pfnCodec);
extern void InterruptSetADCHandler(PFNISR pfnADC);
extern void InterruptSetDAIHandler(PFNISR pfnDAI);
extern void InterruptSetUART1Handler(PFNISR pfnUART1);
extern void InterruptSetUART2Handler(PFNISR pfnUART2);
extern void InterruptSetKbdHandler(PFNISR pfnKbd);
extern void InterruptSetSSI2Handler(PFNISR pfnSSI2);
extern void InterruptSetParallelHandler(PFNISR pfnParallel);

//
// From kbd.c:
//
extern void KbdRead(char *pcKeysPressed);
extern char KbdGetButton(void);
extern void KbdNoButton(void);

//
// From keypad.c
//
extern void KPRead(char *pcButtonsPressed);
extern char KPGetUserButton(void);
extern void KPNoButton(void);

//
// From lcd.c
//
extern void LCDLine(long lX1, long lY1, long lX2, long lY2, char cColor);
extern void LCDCircle(long lX, long lY, long lRadius, char cColor);
extern void LCDFillCircle(long lX, long lY, long lRadius, char cColor);
extern void LCDPrintChar(char cChar, long lX, long lY, char cColor);
extern void LCDPrintString(char *pcBuffer, long lX, long lY, char cColor);
extern void LCDPrintCharX2(char cChar, long lX, long lY, char cColor);
extern void LCDPrintStringX2(char *pcBuffer, long lX, long lY, char cColor);
extern void LCDColorLine(long lX1, long lY1, long lX2, long lY2, CPixel sColor);
extern void LCDColorCircle(long lX, long lY, long lRadius, CPixel sColor);
extern void LCDColorFillCircle(long lX, long lY, long lRadius, CPixel sColor);
extern void LCDColorPrintChar(char cChar, long lX, long lY, CPixel sColor);
extern void LCDColorPrintString(char *pcBuffer, long lX, long lY, CPixel sColor);
extern void LCDColorPrintCharX2(char cChar, long lX, long lY, CPixel sColor);
extern void LCDColorPrintStringX2(char *pcBuffer, long lX, long lY, CPixel sColor);


//
// From lcd_optrex.c
//
extern void LCDColorEnable(void);
extern void LCDColorOn(void);
extern void LCDColorOff(void);
extern void LCDColorCls(void);
extern void LCDColorSetPixel(long lX, long lY, CPixel sColor);
extern void LCDColorBacklightOn(void);
extern void LCDColorBacklightOff(void);
extern void LCDColorContrastEnable(void);
extern void LCDColorContrastDisable(void);

//
// From lcd_shrp.c
//
extern void LCDEnable(void);
extern void LCDOn(void);
extern void LCDOff(void);
extern void LCDCls(void);
extern void LCDSetPixel(long lX, long lY, char cColor);
extern char LCDGetPixel(long lX, long lY);
extern void LCDBacklightOn(void);
extern void LCDBacklightOff(void);
extern void LCDContrastEnable(void);
extern void LCDContrastDisable(void);

//
// From nandflsh.c:
//
extern long NANDGetSize(unsigned long *pulDeviceSize,
                        unsigned long *pulPageSize,
                        unsigned long *pulPagesPerBlock,
                        unsigned long *pulNumBlocks);
extern void NANDReadPage(unsigned long ulPage, int bReadSpare,
                         unsigned char *pucBuffer);
extern void NANDEraseBlock(unsigned long ulBlock);
extern void NANDWritePage(unsigned long ulPage, int bWriteSpare,
                          unsigned char *pucBuffer);
extern long SMGetSize(unsigned long *pulDeviceSize, unsigned long *pulPageSize,
                      unsigned long *pulPagesPerBlock,
                      unsigned long *pulNumBlocks);
extern long SMReadPage(unsigned long ulPage, int bReadSpare,
                       unsigned char *pucBuffer);
extern long SMEraseBlock(unsigned long ulBlock);
extern long SMWritePage(unsigned long ulPage, int bWriteSpare,
                        unsigned char *pucBuffer);

//
// From norflsh.c:
//
extern long FlashNumSectors(void);
extern long FlashSectorInfo(long lSector, long *plSectorOffset,
                            long *plSectorLength);
extern void FlashClearLockBits(unsigned long ulFlashBase, long lSector);
extern void FlashEraseChip(unsigned long ulFlashBase);
extern void FlashEraseSector(unsigned long ulFlashBase, long lSector);
extern void FlashProgramBlock(unsigned long ulFlashBase, long lOffset,
                              unsigned char *pucData, long lNumBytes);

//
// From ssi2.c:
//
extern void SSI2Enable(int bMaster);
extern void SSI2Disable(void);
extern void SSI2SendChar(char cChar);
extern char SSI2ReceiveChar(void);

//
// From uart.c:
//
extern long UARTEnable(long lPort, long lDataRate, long lDataBits,
                       long lStopBits, long lParity, long lEvenParity);
extern void UARTDisable(long lPort);
extern void UARTSendChar(long lPort, char cChar);
extern char UARTReceiveChar(long lPort);
extern long UARTCharReady(long lPort);

