
/*
 * $Revision: 1.2 $
 */
 
#include <stdio.h>
#include <stdlib.h>

#include "IntelHex.h"
#include "FileIO.h"

int file_fd;
int (*read_byte_func)(int fd);

static unsigned long curr_load_addr = 0;  /* Where to load memory */
static unsigned long program_start; // Not used.
static unsigned char crc;


static int make_hex()
{
  int ile;
  int c;
  int l = 0;

  for (ile = 0; ile < 2; ile++)
    switch (c = (*read_byte_func)(file_fd))
      {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        
          l = (l << 4) + c - '0';
          break;
          
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        case 'E':
        case 'F':
        
          l = (l << 4) + c - 'A' + 10;
          break;
          
        default:
          FileLogMessage("Bad character in intel hex file.\n");
          FileExit();
      }
  crc += l;
  return (l);
}


void IntelHexProcessFile(int fd,
                         int (*rd_byte_func)(int fd),
                         void (*wr_byte_func)(unsigned long addr, int byte))
{
  unsigned long addr;
  int count;

  enum formatType {standard, segmented, linear} mode;
  unsigned long extendedAddr = 0;
  int recordType;
  
  file_fd = fd;
  read_byte_func = rd_byte_func;

  mode = standard;

  (*rd_byte_func)(fd); //  read in ':'

  for (;;)
    {
      crc = 0;                                   // ':' already read in
      count = make_hex();                        // record length
      addr = (unsigned long)make_hex() << 8;     // high offset 
      addr += make_hex();                        // low offset
      recordType = make_hex();                   // record type
      
      switch (recordType)                        // record type
        {
          case 0:                                // data
          
            if (mode != segmented)
              {
                addr += extendedAddr;

                if (addr != curr_load_addr)
                  {
                    curr_load_addr = addr;
                  }

                while (count--)
                  {
                    (*wr_byte_func)(curr_load_addr++, make_hex());
                  }
              }
            else
              {
                unsigned long loadAddr;
                unsigned long byteIndex = 0;

                while (count--)
                  {
                    loadAddr = ((addr + byteIndex) & 0xFFFF)+ extendedAddr;

                    if (loadAddr != curr_load_addr)
                      {
                        curr_load_addr = loadAddr;
                      }

                    (*wr_byte_func)(curr_load_addr++, make_hex());

                    byteIndex++;
                  }
              }
            break;
          
          case 1:                                // eof
          
//            GeDbgAssert(count == 0);

            (*wr_byte_func)(0, -1); // Flush remaining buffered data.
            if (mode == standard)
              {
                program_start = addr;
              }

            return;
            
          case 2:                                // extended segment address
          
//            GeDbgAssert(mode != linear);
//            GeDbgAssert(count == 2 && addr == 0);

            mode = segmented;

            extendedAddr = (unsigned long)make_hex() << 12;
            extendedAddr += (unsigned long)make_hex() << 4;
            break;
      
          case 3:                                // start segment address

//            GeDbgAssert(mode != linear);
//            GeDbgAssert(count == 4 && addr == 0);

            mode = segmented;

            program_start = (unsigned long)make_hex() << 24;
            program_start += (unsigned long)make_hex() << 16;
            program_start += (unsigned long)make_hex() << 8;
            program_start += (unsigned long)make_hex();
            break;
      
          case 4:                                // extended linear address

//            GeDbgAssert(mode != segmented);
//            GeDbgAssert(count == 2 && addr == 0);

            mode = linear;
            extendedAddr = (unsigned long)make_hex() << 24;
            extendedAddr += (unsigned long)make_hex() << 16;
            break;
      
          case 5:                                // start linear address

//            GeDbgAssert(mode != segmented);
//            GeDbgAssert(count == 4 && addr == 0);

            mode = linear;
            program_start = (unsigned long)make_hex() << 24;
            program_start += (unsigned long)make_hex() << 16;
            program_start += (unsigned long)make_hex() << 8;
            program_start += (unsigned long)make_hex();
            break;
      
          default:
            FileLogMessage("Bad record in intel hex file.\n");
            FileExit();
        }

      make_hex();     /* Read in checksum */

      if (crc)
        {
          FileLogMessage("CRC error in intel hex file.\n");
          FileExit();
        }
        
      while ((*rd_byte_func)(file_fd) != ':')
        ;
    }

}

