
/*
 * $Revision: 1.2 $
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <yfuns.h>

#include "Config.h"
#include "Interface.h"
#include "Private.h"

WriteFunctionType writeByteFunction; // Write byte function pointer.
int overrideProgressBar = 0;         // To disable framework progress bar.

static int messageLogFile = -1;
static int progressBarFile = -1;
static unsigned char* readBuffer;
static int readBufferSize;
static unsigned char* bufp;


void FlRegisterWriteFunction(void (*write_func)(unsigned long address, int byte))
{
  writeByteFunction = write_func;
}


const char* FlFindOption(char* option, int with_value, int argc, char const* argv[])
{
  int i;
  
  for (i = 0; i < argc; i++)
  {
    if (strcmp(option, argv[i]) == 0)
    {
      if (with_value)
      {
        if (i + 1 < argc)
          return argv[i + 1]; // The next argument is the value.
        else
          return 0; // The option was found but there is no value to return.
      }
      else
      {
        return argv[i]; // Return the flag argument itself just to get a non-zero pointer.
      }
    }
  }
  return 0;
}


int FlMakeArgs(char* args, char const* argv[])
{
  int i = 0;
  char* argp;

  // Split the argument string into individual arguments.
  // The arguments are separated by spaces and/or tabs.
  while (argp = strtok(args, " \t"))
  {
    argv[i++] = argp;
    args = 0; // For subsequent strtok calls.
  }
  return i;
}


void FlMessageBox(char* msg)
{
  int log_fd;
  
  log_fd = __open("$MESSAGE_BOX$", _LLIO_CREAT | _LLIO_TRUNC | _LLIO_WRONLY);
  if (log_fd == -1)
  {
    FlErrorExit(); // Something went really wrong, give up.
  }
  
  __write(log_fd, (unsigned char *)msg, strlen(msg));
  __close(log_fd);
}


void FlMessageLog(char* msg)
{
  if (messageLogFile == -1)
  {
    messageLogFile = __open("$DEBUG_LOG$", _LLIO_CREAT | _LLIO_TRUNC | _LLIO_WRONLY);
    if (messageLogFile == -1)
    {
      FlMessageBox("Failed to open message log.");
      FlErrorExit();
    }
  }

  __write(messageLogFile, (unsigned char *)msg, strlen(msg));
}


void FlProgressBarCreate(char* title)
{
  char name[256];

  if (strlen(title) > 240)
  {
    FlMessageBox("Progress bar title is too long (max 240).");
    FlErrorExit();
  }

  strcpy(name, "$PROGRESS_BAR$");
  strcat(name, title);
  if (progressBarFile == -1)
  {
    progressBarFile = __open(name, _LLIO_CREAT | _LLIO_TRUNC | _LLIO_WRONLY);
    if (progressBarFile == -1)
    {
      FlMessageBox("Failed to open progress bar.");
      FlErrorExit();
    }
  }
}


void FlProgressBarDestroy()
{
  if (progressBarFile != -1)
  {
    __close(progressBarFile);
    progressBarFile = -1;
  }
}


void FlProgressBarUpdate(int progress)
{
  unsigned char value = progress;
  
  if (progressBarFile == -1)
  {
    FlMessageBox("Progress bar not open.");
    FlErrorExit();
  }

  __write(progressBarFile, &value, 1);
}


void FlOverrideProgressBar()
{
  overrideProgressBar = 1;
}


void FlErrorExit()
{
  if (messageLogFile != -1)
  {
    __close(messageLogFile);
  }
  
  exit(1);
}


int FlFileOpen(char* name)
{
  int fd;
  extern unsigned char RAMEND; // This symbol is defined in the linker control file.
#pragma segment="HEAP"

  // The read buffer is dynamically allocated to make it as large as possible.
  // This is done by using all unused space in RAM. It is calculated by taking
  // the difference between the end of the last segment (the heap) and the
  // last address of the RAM.
  readBuffer = __segment_end("HEAP");
  readBufferSize = &RAMEND - readBuffer;

  // A small read buffer will slow down performance due to an increased number
  // of JTAG transactions.
  if (readBufferSize < 256)
  {
    FlMessageBox("The read buffer is too small.");
    FlErrorExit();
  }

  fd = __open(name, _LLIO_RDONLY | _LLIO_BINARY);

  return fd;
}


void FlFileClose(int fd)
{
  __close(fd);
}


int FlFileReadByte(int fd)
{                        
  int c;
  static int remaining = 0;

  if (remaining == 0)
  {
    remaining = __read(fd, readBuffer, readBufferSize);
    if (remaining == 0)
    {
      return -1;
    }
    bufp = readBuffer;
  }
  c = *bufp++;
  --remaining;

  return c;
}


