
/*
 * $Revision: 1.2 $
 */
 
#include <stdio.h>
#include <stdlib.h>
#include "Config.h"
#include "SimpleCode.h"
#include "Interface.h"
#include "Private.h"

// C-SPY will automatically setup these variables.
__no_init int __argc;
__no_init const char* __argv[MAX_ARGS];
__no_init __root char __argvbuf[MAX_ARG_SIZE];

static int Main(int argc, char const* argv[]);


int main()
{
#ifdef DEBUG
  __argc = FlMakeArgs(DEBUG_ARGS, __argv);
#endif
  return Main(__argc, __argv); // Call as traditional main() with argc/argv.
}


static int Main(int argc, char const* argv[])
{
  int file_fd;
  const char *str;

  // Check for standard options.
  str = FlFindOption("--flashbase", 1, argc, argv);
  if (str)
  {
    baseAddress = strtoul(str, 0, 0);
    overrideBaseAddress = 1;
  }

  // Initialize flash loader driver.
  FlashDriverInitialize(argc, argv);

  // Verify that the flash loader driver registered the write function.
  if (writeByteFunction == 0)
  {
    FlMessageBox("Flasher must call FlSetWriteByteFunction().");
    FlErrorExit();
  }
  
#ifdef DEBUG
  file_fd = FlFileOpen(DEBUG_FILE);
#else
  // Open the binary image of the application to be flashed.
  // The file must be in the IAR simple-code format.
  file_fd = FlFileOpen("$TARGET_BPATH$.sim");
#endif

  if (file_fd == -1)
  {
    FlMessageBox("Failed to open file $TARGET_BPATH$.sim\n"
                 "\n"
                 "Make sure the following linker options are set:\n"
                 "Output\n"
                 "- Debug information for C-SPY\n"
                 "-   With runtime control modules\n"
                 "-     With I/O emulation modules\n"
                 "-     Allow C-SPY-specific extra output file\n"
                 "Extra Output\n"
                 "- Generate extra output file\n"
                 "-   Output format: simple-code");
    FlErrorExit();
  }

  // Read input file, decode it and flash.
  SimpleCodeProcessFile(file_fd, writeByteFunction);

  FlFileClose(file_fd);
  
  return 0; // Tell C-SPY that the flash download was successful.
}

