;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Common cfi declaratios used in the assembler part of ICCARM's 
;; library.
;;
;;

#define cfi_REG_SIZE    4

CFI_NAME_BLOCK:		MACRO
					cfi names armName_runtime_lib
					cfi StackFrame CFA SP HUGEDATA
					cfi Resource R0:32, R1:32, R2:32, R3:32, R4:32, R5:32, R6:32, R7:32
					cfi Resource R8:32, R9:32, R10:32, R11:32, R12:32, SP:32, LR:32
                    cfi Resource CPSR:32, SPSR:32
					cfi VirtualResource ?RET:32
					cfi EndNames armName_runtime_lib
					ENDM


cfi_COMMON_BLOCK0   MACRO name codeAlign       
					cfi DataAlign 4
					cfi ReturnAddress ?RET CODE
					cfi CFA SP+0
					cfi R0 SameValue
					cfi R1 SameValue
					cfi R2 SameValue
					cfi R3 SameValue
					cfi R4 SameValue
					cfi R5 SameValue
					cfi R6 SameValue
					cfi R7 SameValue
					cfi R8 SameValue
					cfi R9 SameValue
					cfi R10 SameValue
					cfi R11 SameValue
					cfi R12 Undefined
					cfi LR Undefined  
					cfi CPSR SameValue
					cfi SPSR SameValue
					cfi ?RET LR
					ENDM


cfi_COMMON_ARM		MACRO
					cfi Common ARM_runtime_lib using armName_runtime_lib
					cfi CodeAlign 4
					cfi_COMMON_BLOCK0
					cfi EndCommon ARM_runtime_lib
					ENDM
        
cfi_COMMON_Thumb    MACRO
					cfi Common Thumb_runtime_lib using armName_runtime_lib
					cfi CodeAlign 2
					cfi_COMMON_BLOCK0
					cfi EndCommon Thumb_runtime_lib
                    ENDM


;;
;; CFI Macro: cfi_ARM_BLOCK_start name
;;            cfi_ARM_BLOCK_end name
;;			  cfi_TH_BLOCK_start name
;;			  cfi_TH_BLOCK_end name
;;
;; Parameter: name      Name of data block
;;
;; Description: Call Frame Information Data Block
;;

cfi_ARM_BLOCK_start MACRO name
                    cfi block ARMData_\1 using ARM_runtime_lib
                    cfi nofunction
cfi_OFFSET		SET	0
                    ENDM

cfi_ARM_BLOCK_end   MACRO name
                    IF cfi_OFFSET
                      ERROR "Non-zero offset on cfi data block end"
                    ELSE
                      cfi_ARM_BLOCK_endx \1
                    ENDIF
                    ENDM

cfi_ARM_BLOCK_endx  MACRO name
                      cfi endblock ARMData_\1
                    ENDM



cfi_TH_BLOCK_start  MACRO name
                    cfi block ThumbData_\1 using Thumb_runtime_lib
                    cfi nofunction
cfi_OFFSET		SET	0
                    ENDM

cfi_TH_BLOCK_end    MACRO name
                    IF cfi_OFFSET
                      ERROR "Non-zero offset on cfi data block end"
                    ELSE
                      cfi_TH_BLOCK_endx \1
                    ENDIF
                    ENDM

cfi_TH_BLOCK_endx   MACRO name
                    cfi endblock ThumbData_\1
                    ENDM

;;
;; CFI Macro: cfi_STACK_ALLOC bytes
;;
;; Parameter: bytes      Number of bytes to allocate
;;
;; Description: Allocate word on stack
;;

cfi_STACK_ALLOC:    MACRO   bytes
                    IF bytes % cfi_REG_SIZE
                      ERROR "Allocate size not multiple of regsize"
                    ELSE
cfi_OFFSET  SET cfi_OFFSET + bytes
                      cfi             cfa SP + cfi_OFFSET
                    ENDIF
                    ENDM



;;
;; CFI Macro: cfi_STACK_DEALL bytes
;;
;; Parameter: bytes      Number of bytes to deallocate
;;
;; Description: Deallocate word from stack
;;

cfi_STACK_DEALL     MACRO   bytes
                    IF bytes % cfi_REG_SIZE
                      ERROR "Deallocate size not a multiple of regsize"
                    ELSE
cfi_OFFSET  SET cfi_OFFSET - bytes
                      IF cfi_OFFSET < 0
                        ERROR "Negative size on cfi deallocate"
                      ELSEIF cfi_OFFSET
                        cfi           cfa SP + cfi_OFFSET
                      ELSE
                        cfi           cfa SP
                      ENDIF
                    ENDIF
                    ENDM

;;
;; CFI Macro: cfi_PUSH_REG reg offset
;;
;; Parameter: reg       Register to push
;;            offset    Stack offset of pushed register
;;
;; Description: Move reg to stack
;;

cfi_PUSH_REG        MACRO reg, offset
					  cfi_STACK_ALLOC cfi_REG_SIZE
                    IF offset % cfi_REG_SIZE
                      ERROR "Push size not a multiple of regsize"
                    ELSE
                      cfi reg frame(cfa, -cfi_OFFSET + offset)
                    ENDIF
                    ENDM

;;
;; CFI Macro: cfi_SET_REG reg_from, reg_to
;;
;; Parameter: reg_from  Register to move from
;;            reg_to    Register to move to or 'samevalue'
;;
;; Description: Move from REG to REG (samevalue)
;;

cfi_SET_REG         MACRO reg_from, reg_to
                    cfi reg_from reg_to
                    ENDM

;;
;; CFI Macro: cfi_INVALID
;;
;; Description: Set all info invalid
;;

cfi_INVALID:            MACRO
                        cfi invalid
                        ENDM

;;
;; CFI Macro: cfi_VALID
;;
;; Description: Set all info valid
;;

cfi_VALID:              MACRO
                        cfi valid
                        ENDM


