/* Low-level debugger interface. */
/* Copyright (C) 2003  IAR Systems.  All rights reserved. */

/*
 * The interface itself is one empty C function that the debugger breaks on.
 * The function takes an operation-parameter and and a pointer to the operand
 * area. Returns are made via the operand area.
 *
 * A specific target-chip can add operations by taking a number above 127 as
 * operations-id and define an operands struct for the operation. The code in
 * the library should look like:
 *   rettype myFunc(params)
 *   {
 *     __MyDebugParams p;
 *     p.blabla = params;
 *     __DebugBreak(__MyDebugOperation, &p);
 *     return p.returnoperand;
 *   }
 *
 * The code in the debugger C-SPY will be called with __MyDebugOperation and a
 * void pointer (that must be casted to a __MyDebugParams pointer).
 */

#ifndef __DBG__H
#define __DBG__H

#include <yvals.h>

#define __DebugIllegal       0

#define __Debug__Exit        1
#define __DebugGetenv        2
#define __DebugSystem        3
#define __DebugTime          4
#define __DebugMessage       5


#define __DebugClose         8
#define __DebugOpen          9
#define __DebugRead         10
#define __DebugWrite        11
#define __DebugLseek        12
#define __DebugRemove       13
#define __DebugRename       14
#define __DebugClock        15
#define __DebugAbort        16
#define __DebugReportAssert 17

/* Operation numbers upto 127 are reserved for generic use. Specific products
   can use numbers above 127. */

_C_STD_BEGIN
_C_LIB_DECL
void __DebugBreak(unsigned char operation, void *operands);
void __message(int messagenr);
_END_C_LIB_DECL
_C_STD_END

#endif /* __DBG__H */
