#include <iutility>
#include <stdint.h>
#include <limits.h>

#pragma diag_suppress = Pe236

#define S_(x) do { x; } while (0)
#define COPY(f, l , d, T) \
  S_({ T * ff = (T *) f;  \
       T * ll = (T *) l;  \
       T * dd = (T *) d;  \
       while (ff != ll)   \
       {                  \
         *dd++ = *ff++;   \
       }                  \
       d = dd;            \
     })
#define COPY_BACKWARD(f, l , d, T) \
  S_({ T * ff = (T *) f;  \
       T * ll = (T *) l;  \
       T * dd = (T *) d;  \
       while (ff != ll)   \
       {                  \
         *--dd = *--ll;   \
       }                  \
       d = dd;            \
     })

#ifdef INT16_MAX
#define COPY_16_TYPE int16_t
#else /* !INT16_MAX */
#define COPY_16_TYPE char
#endif /* INT16_MAX */

#ifdef INT32_MAX
#define COPY_32_TYPE int32_t
#else /* !INT32_MAX */
#define COPY_32_TYPE char
#endif /* INT32_MAX */

#ifdef INT64_MAX
#define COPY_64_TYPE int64_t
#else /* !INT64_MAX */
#define COPY_64_TYPE char
#endif /* INT64_MAX */

#define DOCOPY(f, l, d, n)                                        \
  S_( if      ((n) * CHAR_BIT == 16) COPY(f, l, d, COPY_16_TYPE); \
      else if ((n) * CHAR_BIT == 32) COPY(f, l, d, COPY_32_TYPE); \
      else if ((n) * CHAR_BIT == 64) COPY(f, l, d, COPY_64_TYPE); \
      else                           COPY(f, l, d, char);         \
    )

#define DOCOPY_BACKWARD(f, l, d, n)                                        \
  S_( if      ((n) * CHAR_BIT == 16) COPY_BACKWARD(f, l, d, COPY_16_TYPE); \
      else if ((n) * CHAR_BIT == 32) COPY_BACKWARD(f, l, d, COPY_32_TYPE); \
      else if ((n) * CHAR_BIT == 64) COPY_BACKWARD(f, l, d, COPY_64_TYPE); \
      else                           COPY_BACKWARD(f, l, d, char);         \
    )

void *
_Copy_a2(void const *_F, void const *_L, void *_D)
{
  DOCOPY(_F, _L, _D, 2);
  return _D;
}

void *
_Copy_a4(void const *_F, void const *_L, void *_D)
{
  DOCOPY(_F, _L, _D, 4);
  return _D;
}

void *
_Copy_a8(void const *_F, void const *_L, void *_D)
{
  DOCOPY(_F, _L, _D, 8);
  return _D;
}

void *
_Copy_backward_a2(void const *_F, void const *_L, void *_D)
{
  DOCOPY_BACKWARD(_F, _L, _D, 2);
  return _D;
}

void *
_Copy_backward_a4(void const *_F, void const *_L, void *_D)
{
  DOCOPY_BACKWARD(_F, _L, _D, 4);
  return _D;
}

void *
_Copy_backward_a8(void const *_F, void const *_L, void *_D)
{
  DOCOPY_BACKWARD(_F, _L, _D, 8);
  return _D;
}
