/* Multibyte function - Utf8 version. */
/* Copyright (C) 2003 IAR Systems.  All rights reserved. */
#include <yvals.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <xmtloc.h>
#include <xwchar.h>

#ifdef _ENCODING_USE_UTF8

_STD_BEGIN

int _EncodingUtf8_Wctomb(char *s, wchar_t wc, _Mbstatet *pst)
{	/* translate widechar to multibyte - Utf8 version. */
  static const mbstate_t initial = {0};

  unsigned char *su = (unsigned char *)s;
  int nextra;

  if (s == 0)
  {	/* set initial state */
    *pst = initial;
    return (0);
  }

#if WCHAR_MAX <= 0xff
  *su++ = (unsigned char)wc;
  return (1);
#else /* WCHAR_MAX <= 0xff */
  if ((wc & (wchar_t)~0x7f) == 0)
  {	/* generate a single byte */
    *su++ = (unsigned char)wc;
    nextra = 0;
  }
  else if ((wc & (wchar_t)~0x7ff) == 0)
  {	/* generate two bytes */
    *su++ = (unsigned char)(0xc0 | wc >> 6);
    nextra = 1;
  }
#if WCHAR_MAX <= 0xffff
  else
  {	/* generate three bytes */
    *su++ = (unsigned char)(0xe0 | (wc >> 12) & 0x0f);
    nextra = 2;
  }
#else /* WCHAR_MAX <= 0xffff */
  else if ((wc & (wchar_t)~0xffff) == 0)
  {	/* generate three bytes */
    *su++ = (unsigned char)(0xe0 | wc >> 12);
    nextra = 2;
  }
  else if ((wc & (wchar_t)~0x1fffff) == 0)
  {	/* generate four bytes */
    *su++ = (unsigned char)(0xf0 | wc >> 18);
    nextra = 3;
  }
  else if ((wc & (wchar_t)~0x3ffffff) == 0)
  {	/* generate five bytes */
    *su++ = (unsigned char)(0xf8 | wc >> 24);
    nextra = 4;
  }
  else
  {	/* generate six bytes */
    *su++ = (unsigned char)(0xfc | (wc >> 30) & 0x03);
    nextra = 5;
  }
#endif /* 0xffff < WCHAR_MAX */

  for (; 0 < nextra; )
    *su++ = (unsigned char)(0x80 | (wc >> 6 * --nextra) & 0x3f);
  return ((char *)su - s);
#endif /* WCHAR_MAX <= 0xff */
}


int _EncodingUtf8_Mbtowc(wchar_t *pwc, const char *s, size_t nin,
                         _Mbstatet *pst)
{	/* translate multibyte to widechar, given locale info - Utf8 version. */
  char state = pst->_State;
  unsigned char *su = (unsigned char *)s;
  wchar_t wc = pst->_Wchar;
  static const mbstate_t initial = {0};

  if (s == 0)
  {	/* set initial state */
    *pst = initial;
    return (0);
  }

#if WCHAR_MAX <= 0xff
  if (nin == 0)
    return (-2);
  else
  {	/* return a single byte */
    if (pwc != 0)
      *pwc = *su;
    return (*su == 0 ? 0 : 1);
  }
#else /* WCHAR_MAX <= 0xff */
  for (; ; ++su, --nin)
  {	/* consume an input byte */
    if (nin == 0)
    {	/* report incomplete conversion */
      pst->_Wchar = wc;
      pst->_State = state;
      return (-2);
    }
    else if (0 < state)
    {	/* fold in a successor byte */
      if ((*su & 0xc0) != 0x80)
      {	/* report invalid sequence */
        errno = EILSEQ;
        return (-1);
      }
      wc = (wc << 6) | (*su & 0x3f);
      --state;
    }
    else if ((*su & 0x80) == 0)
      wc = *su;	/* consume a single byte */
    else if ((*su & 0xe0) == 0xc0)
    {	/* consume first of two bytes */
      wc = *su & 0x1f;
      state = 1;
    }
    else if ((*su & 0xf0) == 0xe0)
    {	/* consume first of three bytes */
      wc = *su & 0x0f;
      state = 2;
    }
#if 0xffff < WCHAR_MAX
    else if ((*su & 0xf8) == 0xf0)
    {	/* consume first of four bytes */
      wc = *su & 0x07;
      state = 3;
    }
    else if ((*su & 0xfc) == 0xf8)
    {	/* consume first of five bytes */
      wc = *su & 0x03;
      state = 4;
    }
    else if ((*su & 0xfc) == 0xfc)
    {	/* consume first of six bytes */
      wc = *su & 0x03;
      state = 5;
    }
#endif /* 0xffff < WCHAR_MAX */
    else
    {	/* report invalid sequence */
      errno = EILSEQ;
      return (-1);
    }
    if (state == 0)
    {	/* produce an output wchar */
      if (pwc != 0)
        *pwc = wc;
      pst->_State = 0;
      return (wc == 0 ? 0 : (const char *)++su - s);
    }
  }
#endif /* WCHAR_MAX <= 0xff */
}

_STD_END

#endif /* _ENCODING_USE_UTF8 */

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
