/* _Ldtob function */
#include <yvals.h>
#if _DLIB_PRINTF_SPECIFIER_FLOAT

#include <float.h>
#include <stdlib.h>
#include <string.h>
#include "xmath.h"
#include "xstdio.h"
_STD_BEGIN

		/* macros */
#define MAXDIG	40	/* safe for 128-bit long double */
#define NDIG	8	/* decimal digits generated for each multiply */
#define NXDIG	7	/* hexadecimal digits generated for each multiply */

#ifdef __IAR_SYSTEMS_ICC__
static long double scale(long double ldval, int mask)
{
  long double factor;

  for(factor = 10.0; mask != 0; mask >>= 1, factor *= factor)
    if (mask & 1)
      ldval *= factor;

  return ldval;
}

#if _DLIB_PRINTF_SPECIFIER_A

static char a2dig(char c, char code)
{
  c += '0';
  if (c > '9')
    c += (code - 'a') + 'a' - '9' - 1;

  return c;
}

#endif /* _DLIB_PRINTF_SPECIFIER_A */

#else /* !__IAR_SYSTEMS_ICC__ */
		/* static data */
static const long double pows[] = {
	1e1L, 1e2L, 1e4L, 1e8L, 1e16L, 1e32L,
#if 0x100 < _LBIAS	/* assume IEEE 754 8- or 10-byte */
	1e64L, 1e128L, 1e256L,
#if _DLONG			/* assume IEEE 754 10-byte */
	1e512L, 1e1024L, 1e2048L, 1e4096L,
#endif
#endif
	};
#endif /* __IAR_SYSTEMS_ICC__ */

void _Ldtob(_Pft *px, char code)
{	/* convert long double to text */
  char ac[MAXDIG];
  char *p = ac;
  long double ldval = px->v.ld;
  short errx, nsig, xexp;

#if _DLIB_PRINTF_SPECIFIER_A
  if (_LC(code) == 'a')
    ;
  else
#endif
#if _DLIB_PRINTF_WIDTH_AND_PRECISION
  if (px->prec < 0)
    px->prec = 6;
  else if (px->prec == 0 && _LC(code) == 'g')
    px->prec = 1;
#endif
  if ((errx = _L_FNAME(Dunscale)(&xexp, _L_PTRCAST &px->v.ld)) == _NANCODE)
  {	/* x == NaN */
    memcpy(px->s, (code >= 'a' && code <= 'z') ? "nan" : "NAN", px->n1 = 3);
    return;
  }
  else if (0 < errx)
  {	/* x == INF */
    memcpy(px->s, (code >= 'a' && code <= 'z') ? "inf" : "INF", px->n1 = 3);
    return;
  }

#if _DLIB_PRINTF_SPECIFIER_A
  if (_LC(code) == 'a')
  {	/* put "0x" */
    *px->s++ = '0';
    *px->s++ = code == 'a' ? 'x' : 'X';
    px->n0 +=2;
  }
#endif

  if (0 == errx)	/* x == 0 */
    nsig = 0, xexp = 0;
#if _DLIB_PRINTF_SPECIFIER_A
  else if (_LC(code) == 'a')
  {	/* 0 < |x|, generate hex fraction, binary exponent */
#ifndef __IAR_SYSTEMS_ICC__
    const char *digits = code == 'a'
      ? "0123456789abcdef" : "0123456789ABCDEF";
#endif /* !__IAR_SYSTEMS_ICC__ */
    int gen;

#if _DLIB_PRINTF_WIDTH_AND_PRECISION
    nsig = px->prec < 0 ? MAXDIG - NXDIG : px->prec + 1;
#else
    nsig = 1;
#endif
    gen = nsig + 1;
    ldval = ldval < 0 ? -px->v.ld : px->v.ld;
    xexp -= 4;	/* one leading nonzero hex digit */

    for (*p++ = 0x0; 0 < gen && 0 < ldval; p += NXDIG)
    {	/* convert NXDIG at a time */
      int j;
      long lo;

      _L_FNAME(Dscale)(_L_PTRCAST &ldval, 4 * NXDIG);
      lo = (long)ldval;
      if (0 < (gen -= NXDIG))
        ldval -= (long double)lo;
      for (p += NXDIG, j = NXDIG; 0 < lo && 0 <= --j; )
        *--p = (int)(lo & 0xf), lo >>= 4;
      while (0 <= --j)
        *--p = 0;
    }
    gen = p - &ac[1];
    p = &ac[1];
    if (gen < nsig)
      nsig = gen;
    if (0 <= nsig)
    {	/* round and strip trailing zeros */
      const char drop = nsig < gen && 0x8 <= p[nsig] ? 0xf : 0x0;
      int n;

      for (n = nsig; p[--n] == drop; )
        --nsig;
      if (drop == 0xf)
        ++p[n];
      if (n < 0)
        --p, ++nsig, xexp += 4;
      for (n = nsig; 0 <= --n; )
#ifdef __IAR_SYSTEMS_ICC__
        p[n] = a2dig(p[n], code);
#else /* !__IAR_SYSTEMS_ICC__ */
        p[n] = digits[p[n]];
#endif /* __IAR_SYSTEMS_ICC__ */
    }
#if _DLIB_PRINTF_WIDTH_AND_PRECISION
    if (px->prec < 0)
      px->prec = nsig - 1;
#endif
  }
#endif /* _DLIB_PRINTF_SPECIFIER_A */
  else
  {	/* 0 < |x|, generate decimal fraction and exponent */
    {	/* scale ldval to ~~10^(NDIG/2) */
#ifdef __IAR_SYSTEMS_ICC__
      int n;

      if (ldval < 0)
        ldval = -ldval;
      xexp = xexp * 30103L / 100000L;
      n = NDIG - xexp - 1;
      if (n > 0)
        ldval = scale(ldval, n);
      else
        ldval /= scale(1.0, -n);
#else /* !__IAR_SYSTEMS_ICC__ */
      int i, n;

      if (ldval < 0)
        ldval = -ldval;
      if ((xexp = xexp * 30103L / 100000L - NDIG / 2) < 0)
      {	/* scale up */
        n = (-xexp + (NDIG / 2 - 1)) & ~(NDIG / 2 - 1);
        xexp = -n;
        for (i = 0; 0 < n; n >>= 1, ++i)
          if (n & 1)
            ldval *= pows[i];
      }
      else if (0 < xexp)
      {	/* scale down */
        long double factor = 1.0;

        xexp &= ~(NDIG / 2 - 1);
        for (n = xexp, i = 0; 0 < n; n >>= 1, ++i)
          if (n & 1)
            factor *= pows[i];
        ldval /= factor;
      }
#endif /* __IAR_SYSTEMS_ICC__ */
    }
    {	/* convert significant digits */
#if _DLIB_PRINTF_WIDTH_AND_PRECISION
      int gen = px->prec + (_LC(code) == 'f'
                            ? xexp + 2 + NDIG : 2 + NDIG / 2);
#else
      int gen = ((_LC(code) == 'g') ? 1 : 0) +
                ((_LC(code) == 'f' ? xexp + 2 + NDIG : 2 + NDIG / 2));
#endif

      if (LDBL_DIG + NDIG / 2 < gen)
        gen = LDBL_DIG + NDIG / 2;
#ifdef __IAR_SYSTEMS_ICC__
      *p++ = '0';
      while(gen > 0)
      {
        unsigned long lo = (unsigned long)ldval;
        signed char digcount;

        for(digcount = NDIG, p += NDIG; digcount > 0; --digcount)
        {
          char ch = lo % 10;
          lo /= 10;
          *--p = '0' + ch;
        }

        p += NDIG;
        gen -= NDIG;
        if (gen > 0)
          ldval = (ldval - (unsigned long)ldval) * 1e8L;
      }
#else /* !__IAR_SYSTEMS_ICC__ */
      for (*p++ = '0'; 0 < gen && 0 < ldval; p += NDIG)
      {	/* convert NDIG at a time */
        int j;
        long lo = (long)ldval;

        if (0 < (gen -= NDIG))
          ldval = (ldval - (long double)lo) * 1e8L;
        for (p += NDIG, j = NDIG; 0 < lo && 0 <= --j; )
        {	/* convert NDIG digits */
          ldiv_t qr = ldiv(lo, 10);

          *--p = qr.rem + '0', lo = qr.quot;
        }
        while (0 <= --j)
          *--p = '0';
      }
#endif /* __IAR_SYSTEMS_ICC__ */

      gen = p - &ac[1];
#ifdef __IAR_SYSTEMS_ICC__
      for (p = &ac[1]; *p == '0'; ++p)
#else /* !__IAR_SYSTEMS_ICC__ */
      for (p = &ac[1], xexp += NDIG-1; *p == '0'; ++p)
#endif /* __IAR_SYSTEMS_ICC__ */
        --gen, --xexp;	/* correct xexp */
#if _DLIB_PRINTF_WIDTH_AND_PRECISION
      nsig = px->prec + (_LC(code) == 'f'
                         ? xexp + 1 : _LC(code) == 'e' ? 1 : 0);
#else
      nsig = ((_LC(code) == 'g') ? 1 : 0) +
             ((_LC(code) == 'f'
                         ? xexp + 1 : _LC(code) == 'e' ? 1 : 0));
#endif
      if (gen < nsig)
        nsig = gen;
      if (0 <= nsig)
      {	/* round and strip trailing zeros */
        const char drop = nsig < gen && '5' <= p[nsig] ? '9' : '0';
        int n;

        for (n = nsig; p[--n] == drop; )
          --nsig;
        if (drop == '9')
          ++p[n];
        if (n < 0)
          --p, ++nsig, ++xexp;
      }
    }
  }
  _Genld(px, code, p, nsig, xexp);
}
_STD_END

#endif /* _DLIB_PRINTF_SPECIFIER_FLOAT */
/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
